/*! \file "dos.h" \brief DOS-Definitions */
///	\ingroup multifat
///	\defgroup DOS DOS-Functions (dos.h)
//#########################################################################
// Benutzung auf eigene Gefahr !
//
// Use at your own risk !
//
// 18.02.2007 Changed F_READ to 'r', F_WRITE to 'w'
//            Now you can do Fopen("mydata.txt",'w');    ;)
//
//#########################################################################
// Last change: 23.08.2007
//#########################################################################
// Compiler: AVR-GCC 4.1.1
//#########################################################################
//@{
#ifndef __DOS_H
#define __DOS_H

#define MAX_OPEN_FILE   ((char) 3) ///< Declare how many files you want to open at the same time
// every 1 declared open file takes 558 Bytes for FAT12/16/32
// every 1 declared open file takes 554 Bytes for FAT12/16 only
//
// ATMega32 has only 2kB RAM. If you want to use 2 open files, you 
// have to switch off the FAT buffer !  

// Do it in dos.h or make an avrtypes.h and include it here
 typedef unsigned char 	U8;  // one byte
 typedef char 			S8;  // one byte signed
 typedef unsigned int 	U16; // two bytes
 typedef int 			S16; // two bytes signed
 typedef unsigned long 	U32; // four bytes
 typedef long 			S32; // four bytes signed

// Some more defines for special filesystem handling

#define STRICT_FILESYSTEM_CHECKING // If you define this, some very rare special cases will be
				    // noticed. But this needs more code.
				    // Special cases when files where made on a Win system and
				    // copied to the flash card:
				    //
                                    // Bug found by Michele Ribaudo
				    // Files with zero filesize have no clusters allocated !
				    // Calling Remove() and Fopen() may hang up the system.
				    // If you define STRICT_FILESYSTEM_CHECKING opening a
				    // zero length file for reading or writing gives an error.
				    // You can remove a zero length file !  
				    //
				    // You don't need this define if all files where made with
				    // my FAT system. Even if filesize is zero.
                                    // If unsure keep it defined !

//fopen flags
#define F_CLOSED	0
#define F_READ		'r' // read only
#define F_WRITE		'w' // read/write, no automatic seek to end of file 
#define F_APPEND	'a' // read/write, automatic seek to end of file

#define F_ERROR		0 // dir/file operation failed
#define F_OK		1 // dir/file operation successfull

#ifndef SEEK_SET
 #define	SEEK_SET	0	/* set file offset to offset */
#endif
#ifndef SEEK_CUR
 #define	SEEK_CUR	1	/* set file offset to current plus offset */
#endif
#ifndef SEEK_END
 #define	SEEK_END	2	/* set file offset to EOF plus offset */
#endif

// #undef defines below in "dosdefs.h" if you don't need them
// spare program memory by deciding if we want to read, write or both
#define DOS_READ	//define this if you want to read files
#define DOS_WRITE	//define this if you want to write files
#define DOS_DELETE	//define this if you want to delete files
#define DOS_READ_RAW	//define this if you want to read files with ReadFileRaw()

#define DOS_CHDIR	//define this if you want to go into subdirectories
#define DOS_MKDIR	//define this if you want to make subdirectories
#define DOS_RENAME	//define this if you want to rename files
#define DOS_FSEEK       //define this if you want to seek in files (reading only)

// spare program memory by deciding if we want to use FAT12, FAT16, FAT32.
// don't touch if you don't know the FAT type of your drive !
//#define USE_FAT12	//define this if you want to use FAT12
//#define USE_FAT16	//define this if you want to use FAT16
#define USE_FAT32	//define this if you want to use FAT32

#define USE_FATBUFFER	//define this if you want to use a FAT buffer
                        //needs 517 Bytes of RAM ! 

#define USE_FINDFILE	//define this if you want to use Findfirst(); Findnext();
#define USE_FINDLONG    //define this if you want to get long filenames
			//from Findfirst(); Findnext();

#define USE_DRIVEFREE   //define this if you want to get free and used space of your drive

#include "dosdefs.h"  // keep the line at this place ! don't move down or delete 

extern char Fopen(char *name, U8 flag);
extern void Fclose(char fileid);

extern U16 Fread(U8 *buf, U16 count, char fileid );
extern U16 Fwrite(U8 *buf, U16 count, char fileid);

extern U8 Fseek(S32 offset, U8 mode, char fileid);
extern void FlushWriteBuffer(char fileid);

extern void Fflush(char fileid);
extern void fflush_all(void);
extern U8 Remove(char *name);
extern U8 Rename(char *OldName, char *NewName);
extern U32 Filelength(char fileid);

extern U8 ReadFileRaw(char *name);

extern U8 FindName(char fileid);

extern char findfreefiledsc(void); // return -1 if too many open files

#ifndef BYTE_PER_SEC
 #define BYTE_PER_SEC	(U16) 512
#endif

struct FileDesc { //FileDescriptor structure
                 U8 iob[BYTE_PER_SEC];      //file i/o buffer
//		 U32 FileCurrentSector;      //number of sector with last data read/written
		 U32 FileDirSector;          //dir sector holding this fileentry
		 U32 FileSize;               
		 U32 FilePosition;           //file byte position 
                 U32 FileFirstClusterSector;    // first sector of cluster with last data read/written
		 U8 FileClusterSectorOffset;   //sector of current cluster used
		 U8 FileFlag;               //open or closed
		 U8 FileAttr;               //file attribute. also used for directory functions
		 U32 FileClusterCount;       //this is NOT uint !
#ifdef USE_FAT32
                 U32 FileFirstCluster;       //needed for UpdateFileEntry() !
                 U32 FileCurrentCluster;     //number of cluster in use
#else //#ifdef USE_FAT32
                 U16 FileFirstCluster;
                 U16 FileCurrentCluster;
#endif //#ifdef USE_FAT32
		 char FileName[11];            //file name
		 U8 FileDirOffset;          //dir entry offset in FileDirSector/32
#ifdef DOS_WRITE
                 U8 FileWriteBufferDirty;
#endif
                };

extern struct FileDesc FileDescriptors[];

//this is for easier and faster converting from byte arrays to UINT, ULONG
//ui and ul share the same memory space

union Convert {
 U16  ui;
 U32 ul;
};


#ifdef MMC_CARD_SPI
 #include "mmc_spi.h"
#endif

#include "fat.h"
#include "dir.h"

#ifdef USE_FINDFILE
 #include "find_x.h"
#endif

#ifdef USE_DRIVEFREE
 #include "drivefree.h"
#endif

#endif //__DOS_H
//@}
