/*************************************************************************************

	Project: 	AVR based Webserver with WIZ810MJ
	Programm:	SD Card Test Program
	Author:		Robert Schilling, Stefan Tschiggerl
	E-Mail:		robert.schilling.aah04@htl-kaindorf.ac.at
				stefan.tschiggerl.aah04@htl-kaindorf.ac.at
	Date:		28.03.2008

***************************************************************************************/

#include <avr/io.h>
#include <string.h>

#include "./Uart/rprintf.h"
#include "./Uart/buffer.h"
#include "./Uart/uart.h"

#include "./SD_Karte/dos.h"
#include "./SD_Karte/dir.h"

/* Definitions for CAN CS to enable the Voltage Converter for SD CArd */
 
#define CAN_CS_DDR		DDRB
#define CAN_CS_PORT		PORTB
#define CAN_CS_PIN		PB4

void initExtmem(void) __attribute__ ((naked)) __attribute__ ((section (".init0")));

void initExtmem(void)
{
	/*Init XMEM Interface */
	MCUCR |= (1<<SRE);
}

void writeTestFile(void)
{
	unsigned char wrote;
	char file_id0;
	
	file_id0 = Fopen("sdWrite.txt", F_WRITE);
	
	if(file_id0 >= 0)
		rprintf("Opened sdWrite.txt\n");
	else
	{
		rprintf("Open sdWrite.txt Failed!\n");
		while(1);
	}
	
	wrote = Fwrite("SD Card Write Test\n"\
				   "Wrote File sdWrite.txt\n"\
				   "Contains 52 Letters\n", 62, file_id0);
	if(wrote != 62)
	{
		rprintf("Write sdWrite.txt Failed\n");
		rprintf("%d letters written\n", wrote);
		while(1);
	}
	rprintf("Write Test OK\n");
	rprintf("Wrote 62 letters\n");
	
	Fclose(file_id0);
	rprintf("Closed sdWrite.txt");
}

int main(void)
{
	/* File Handle for SD Card */
	char file_id0;
	char readBuffer[100];

	
	/* Set Leds to Output, Clear Leds */
	DDRG |= (1<<PG4);
	PORTG &= ~(1<<PG4);
	DDRG |= (1<<PG3);
	PORTG &= ~(1<<PG3);
	
	/* Init Uart an set Baudrate to 9600 Bps */
	uartInit();
	uartSetBaudRate(115200);
	
	/* Init rprintf Lib */
	rprintfInit(uartSendByte);
	
	rprintf("\nSD Card for Avr based Webserver\n");
	rprintf("===============================\n\n");
	
	/* Init SD Card */
	MMC_IO_Init();
	CAN_CS_DDR |= (1 << CAN_CS_PIN);	//Set CAN_CS Output
	CAN_CS_PORT |= (1 << CAN_CS_PIN);
	rprintf("SD Card IO Init OK\n");
	
	if(GetDriveInformation() != F_OK)
	{
		rprintf("Gettind Drive Information Failed!\n");
		while(1);
	}
	
	rprintf("Getting Drive Information OK\n");
	
	file_id0 = Fopen("sdRead.txt", F_READ);
	
	if(file_id0 >= 0)
		rprintf("Opened sdRead.txt\n");
	else
	{
		rprintf("Open sdRead.txt Failed!\n");
		while(1);
	}
	
	rprintf("Filesize: %d\n", Filelength(file_id0));
	
	if(Fread(readBuffer, 43, file_id0) != 43)
	{
		rprintf("Read sdRead.txt Failed\n");
		while(1);
	}
	
	rprintf("Read sdRead.txt OK\n");
	rprintf("%s",readBuffer);		//'\0' steht in der Textdatei
	Fclose(file_id0); 
	rprintf("Closed sdRead.txt"); 
	
	writeTestFile();

	if(Mkdir("temp") != F_OK)
	{
		rprintf("Making Directory Failed!\n");
		while(1);
	}
	
	rprintf("Making directory OK\n");
	
	if(Chdir("temp") != F_OK)
	{
		rprintf("Changing Directory Failed!\n");
		while(1);
	}
	
	rprintf("Changing Directory OK\n");
	writeTestFile();
	
	rprintf("SD Card Test Complete");
	
	while(1);
}
